@@ -50,7 +50,7 @@ class ActionModule(object):
                 result = dict(failed=True, msg=args['msg'])
             else:
                 result = dict(msg=args['msg'])
-        elif 'var' in args:
+        elif 'var' in args and not utils.LOOKUP_REGEX.search(args['var']):
             results = template.template(self.basedir, "{{ %s }}" % args['var'], inject)
             result[args['var']] = results
 
