@@ -23,11 +23,13 @@ import types
 import pipes
 import glob
 import re
+import collections
 import operator as py_operator
 from ansible import errors
 from ansible.utils import md5s
 from distutils.version import LooseVersion, StrictVersion
 from random import SystemRandom
+from jinja2.filters import environmentfilter
 
 def to_nice_yaml(*a, **kw):
     '''Make verbose, human readable yaml'''
@@ -132,6 +134,10 @@ def search(value, pattern='', ignorecase=False):
 
 def regex_replace(value='', pattern='', replacement='', ignorecase=False):
     ''' Perform a `re.sub` returning a string '''
+
+    if not isinstance(value, basestring):
+        value = str(value)
+
     if ignorecase:
         flags = re.I
     else:
@@ -140,19 +146,42 @@ def regex_replace(value='', pattern='', replacement='', ignorecase=False):
     return _re.sub(replacement, value)
 
 def unique(a):
-    return set(a)
+    if isinstance(a,collections.Hashable):
+        c = set(a)
+    else:
+        c = []
+        for x in a:
+            if x not in c:
+                c.append(x)
+    return c
 
 def intersect(a, b):
-    return set(a).intersection(b)
+    if isinstance(a,collections.Hashable) and isinstance(b,collections.Hashable):
+        c = set(a) & set(b)
+    else:
+        c = unique(filter(lambda x: x in b, a))
+    return c
 
 def difference(a, b):
-    return set(a).difference(b)
+    if isinstance(a,collections.Hashable) and isinstance(b,collections.Hashable):
+        c = set(a) - set(b)
+    else:
+        c = unique(filter(lambda x: x not in b, a))
+    return c
 
 def symmetric_difference(a, b):
-    return set(a).symmetric_difference(b)
+    if isinstance(a,collections.Hashable) and isinstance(b,collections.Hashable):
+        c = set(a) ^ set(b)
+    else:
+        c = unique(filter(lambda x: x not in intersect(a,b), union(a,b)))
+    return c
 
 def union(a, b):
-    return set(a).union(b)
+    if isinstance(a,collections.Hashable) and isinstance(b,collections.Hashable):
+        c = set(a) | set(b)
+    else:
+        c = unique(a + b)
+    return c
 
 def version_compare(value, version, operator='eq', strict=False):
     ''' Perform a version comparison on a value '''
@@ -181,7 +210,8 @@ def version_compare(value, version, operator='eq', strict=False):
     except Exception, e:
         raise errors.AnsibleFilterError('Version comparison: %s' % e)
 
-def rand(end, start=None, step=None):
+@environmentfilter
+def rand(environment, end, start=None, step=None):
     r = SystemRandom()
     if isinstance(end, (int, long)):
         if not start:
@@ -220,6 +250,7 @@ class FilterModule(object):
             'dirname': os.path.dirname,
             'expanduser': os.path.expanduser,
             'realpath': os.path.realpath,
+            'relpath': os.path.relpath,
 
             # failure testing
             'failed'  : failed,
