@@ -15,6 +15,10 @@
 # You should have received a copy of the GNU General Public License
 # along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
 
+from ansible.utils import safe_eval
+import ansible.utils as utils
+import ansible.errors as errors
+
 def flatten(terms):
     ret = []
     for term in terms:
@@ -26,10 +30,15 @@ def flatten(terms):
 
 class LookupModule(object):
 
-    def __init__(self, **kwargs):
-        pass
+    def __init__(self, basedir=None, **kwargs):
+        self.basedir = basedir
+
+    def run(self, terms, inject=None, **kwargs):
+        terms = utils.listify_lookup_plugin_terms(terms, self.basedir, inject) 
+
+        if not isinstance(terms, list) and not isinstance(terms,set):
+            raise errors.AnsibleError("with_items expects a list or a set")
 
-    def run(self, terms, **kwargs):
-        if isinstance(terms, basestring):
-            terms = [ terms ]
         return flatten(terms)
+
+
